from oauth2client.service_account import ServiceAccountCredentials
import pulumi
from googleapiclient.discovery import build


def get_key_file_location():
    """Returns google key file location"""
    config = pulumi.Config()
    return config.require("google_api_key_file")


def get_service(api_name, api_version, scopes, key_location):
    """Create a google service

    :api_name: the Tag Manager service object.
    :api_version: the path of the Tag Manager account from which to retrieve the container
    :scopes: name of the container
    :key_file_location: location of key file

    Returns:
      The google api service
    """
    credentials = ServiceAccountCredentials.from_json_keyfile_name(
        key_location, scopes=scopes
    )
    # Build the service object.
    service = build(api_name, api_version, credentials=credentials)
    return service



from pulumi.dynamic import Resource
from pulumi import Input, Output
from .web_property_provider import WebPropertyProvider
from ..service import get_key_file_location


class WebPropertyArgs(object):
    """
    Describes a GA web property.
    """

    account_id: Input[str]
    """
    Account ID of Google Analytics
    """

    site_name: Input[str]
    """
    Name of the website for this web property.
    """

    site_url: Input[str]
    """
    Website url for this web property.
    """

    def __init__(
        self, account_id, site_name, site_url,
    ):
        self.account_id = account_id
        self.site_name = site_name
        self.site_url = site_url


class WebProperty(Resource):
    tracking_id: Output[str]

    def __init__(self, name, args: WebPropertyArgs, opts=None):
        full_args = {
            "tracking_id": None,
            "key_location": get_key_file_location(),
            **vars(args),
        }
        super().__init__(WebPropertyProvider(), name, full_args, opts)



from pulumi.dynamic import ResourceProvider, CreateResult, UpdateResult
from ..service import get_service

SCOPES = ["https://www.googleapis.com/auth/analytics.edit"]


class WebPropertyProvider(ResourceProvider):
    def create(self, props):
        service = get_service("analytics", "v3", SCOPES, props["key_location"])

        properties = (
            service.management()
            .webproperties()
            .list(accountId=props["account_id"], fields="items")
            .execute()
        )

        if properties.get("items"):
            # check if property already exists then simply return tracking code from property
            for p in properties.get("items"):
                if props["site_name"] == p.get("name"):
                    return CreateResult(
                        id_=props["account_id"], outs={"tracking_id": p["id"], **props},
                    )
        web_property = (
            service.management()
            .webproperties()
            .insert(
                accountId=props["account_id"],
                fields="id",
                body={"websiteUrl": props["site_url"], "name": props["site_name"]},
            )
            .execute()
        )

        return CreateResult(
            id_=props["account_id"], outs={"tracking_id": web_property["id"], **props},
        )

    def update(self, id, _olds, props):
        service = get_service("analytics", "v3", SCOPES, props["key_location"])

        web_property = (
            service.management()
            .webproperties()
            .update(
                accountId=props["account_id"],
                webPropertyId=_olds["tracking_id"],
                body={"websiteUrl": props["site_url"], "name": props["site_url"]},
            )
            .execute()
        )

        return UpdateResult(outs={"tracking_id": web_property["id"], **props})



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pulumi_google_analytics",
    version="0.0.1",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nuage-studio/pulumi-google-analytics",
    packages=find_packages(exclude=('tests', 'example')),
    python_requires='>=3.7',
    install_requires=[
        'google-api-python-client>=1.7.11',
        'pulumi>=1.8.1',
        'pulumi_aws>=1.17.0',
        'oauth2client>=4.1.3'
    ]
)